<?php
if (!defined('ABSPATH')) {
    exit();
}

if (!class_exists('App_Expert_Settings_Controller')) {

    class App_Expert_Settings_Controller extends WP_REST_Post_Types_Controller
    {

        const REMOVE_POST_TYPES = array(
            'wp_block',
            'attachment',
            'page'
        );
        public function __construct()
        {
            $this->rest_base = 'settings';
            $this->namespace = APP_EXPERT_API_NAMESPACE;
        }


        public function register_routes()
        {
            register_rest_route(
                $this->namespace,
                '/' . $this->rest_base,
                array(
                    array(
                        'methods'             => WP_REST_Server::READABLE,
                        'callback'            => array($this, 'get_items'),
                        'args'                => $this->get_collection_params(),
                    ),
                    'schema' => array($this, 'get_public_item_schema'),
                )
            );
        }

        public function get_items($request)
        {
            $is_WPML_enabled = self::is_WPML_enabled();
            $locales = $this->get_languages_array($is_WPML_enabled);
            $data = ['post_types' => []];
            foreach (get_post_types(array(), 'object') as $obj) {
                if (empty($obj->show_in_rest) || array_search($obj->name, self::REMOVE_POST_TYPES, true) !== FALSE || ('edit' === $request['context'] && !current_user_can($obj->cap->edit_posts))) {
                    continue;
                }

                $post_type = $this->prepare_item_for_response($obj, $request);
                if (is_null($post_type)) {
                    continue; // Some required data are missing
                }

                $translated_names = [];
                foreach ($locales as $locale) {
                    $translation_domain = $this->get_post_type_translation_domain($post_type->data['slug']);
                    $translated_names[$locale] = $is_WPML_enabled ?
                        apply_filters('wpml_translate_single_string', $post_type->data['name'], $translation_domain, $post_type->data['name'], $locale)
                        : __($post_type->data['name'], $translation_domain);
                }
                $post_type->data['translated_name'] = $translated_names;
                $data['post_types'][$obj->name] = $this->prepare_response_for_collection($post_type);
            }

            $data = apply_filters('appexpert_settings_data', $data);

            $response =  rest_ensure_response($data);

            if (is_wp_error($response))
                return $response;


            return App_Expert_Response::get_api_list_success_response(
                $response,
                'app_expert_settings_retrieved',
                'retrieved all settings'
            );
        }



        /**
         * Get array of available languages codes
         * 
         * @var bool $is_WPML_enabled
         * 
         * @return array
         */
        public function  get_languages_array($is_WPML_enabled)
        {

            if ($is_WPML_enabled) {
                $langs_data = apply_filters('wpml_active_languages', NULL, 'orderby=id&order=asc');
                $langs = array_column($langs_data, 'code');
            } else {
                $langs = array();
                $locale = get_locale();
                require_once(ABSPATH . 'wp-admin/includes/translation-install.php');
                $translations = wp_get_available_translations();

                if (isset($translations[$locale])) {
                    $translation = $translations[$locale];
                    $lang = current($translation['iso']);
                } else {
                    $lang = $locale;
                }
                $lang = $lang == "en_US" ? "en" : $lang;
                $langs = [$lang];
            }

            return $langs;
        }


        /**
         * Check if the WPML plugin is enabled or not
         * 
         * @return bool
         */
        public static function is_WPML_enabled()
        {
            return in_array('sitepress-multilingual-cms/sitepress.php', apply_filters('active_plugins', get_option('active_plugins')));
        }


        /**
         * Get translation domain for the given post type
         * @var string $post_type Post type slug
         * 
         * @return string domain name
         */
        protected function get_post_type_translation_domain($post_type)
        {
            switch ($post_type):
                case 'product':
                    $domain = 'woocommerce';
                    break;
                case 'post':
                    $domain = 'default';
                    break;
                default:
                    $domain = 'admin_texts_cptui_post_types';
                    break;
            endswitch;

            $domain = apply_filters('appexpert_post_type_translation_domain', $domain, $post_type);
            return $domain;
        }

        /**
         * Prepares a post type object for serialization.
         *
         * @param WP_Post_Type    $post_type Post type object.
         * @param WP_REST_Request $request   Full details about the request.
         * @return WP_REST_Response Response object.
         */
        public function prepare_item_for_response($post_type, $request)
        {

            $fields = $this->get_fields_for_response($request);
            $data   = array();


            // Check all required keys exists
            $required_keys = ['name', 'slug', 'rest_base'];
            if (count(array_intersect($required_keys, array_keys($fields))) == count($required_keys)) {
                return null;
            }

            $data['name'] = $post_type->label;
            $data['slug'] = $post_type->name;
            $data['rest_base'] = $post_type->rest_base ? $post_type->rest_base : $post_type->name;
            $data['taxonomies'] = get_object_taxonomies($post_type->name);



            $context = !empty($request['context']) ? $request['context'] : 'view';
            $data    = $this->add_additional_fields_to_object($data, $request);
            $data    = $this->filter_response_by_context($data, $context);


            return rest_ensure_response($data);
        }
    }
}

$App_Expert_Settings_Controller = new App_Expert_Settings_Controller();
$App_Expert_Settings_Controller->register_routes();
